@@ -10,7 +10,9 @@ from sentence_transformers import SentenceTransformer, util, InputExample, losse
 from string import punctuation
 from nltk.stem import PorterStemmer
 from nltk.stem.isri import ISRIStemmer
-from scipy.spatial.distance import cdist
+from tqdm import tqdm
+from collections import Counter
+download('stopwords')
 
 def preprocess(sentence: str, remove_punct: bool, remove_stop_words: bool, stemm: bool, lang: str='en') -> str: 
     if lang.lower() == 'en':
@@ -20,7 +22,6 @@ def preprocess(sentence: str, remove_punct: bool, remove_stop_words: bool, stemm
             sentence = sentence.translate(str.maketrans('', '', punctuation))
         # remove stop words and stem
         if remove_stop_words and stemm:
-            download('stopwords')
             stop_words = stopwords.words('english')
             return ' '.join([ps.stem(w) for w in sentence.lower().split() if w not in stop_words])
         # stem only
@@ -48,35 +49,15 @@ def preprocess(sentence: str, remove_punct: bool, remove_stop_words: bool, stemm
     else:
         raise Exception('non recognized language please specify either en|ar')
         
-        
-from typing import Union
-from nltk.stem import PorterStemmer
-import numpy as np
-import pandas as pd
-from nltk.corpus import stopwords
-from nltk import download
-from gensim.models import KeyedVectors
-import gensim.downloader as api
-from sentence_transformers import SentenceTransformer, util, InputExample, losses, models
-from string import punctuation
-from nltk.stem import PorterStemmer
-from nltk.stem.isri import ISRIStemmer
-from sklearn.metrics.pairwise import cosine_similarity
-from numba import jit
-from tqdm import tqdm
-import torch 
-from collections import Counter
 
-from time import time
 def preprocess(sentence: str, remove_punct: bool, remove_stop_words: bool, stemm: bool, lang: str='en') -> str: 
     if lang.lower() == 'en':
         ps = PorterStemmer()
         # remove punctuations
-        if not remove_punct:
+        if remove_punct:
             sentence = sentence.translate(str.maketrans('', '', punctuation))
         # remove stop words and stem
         if remove_stop_words and stemm:
-            download('stopwords')
             stop_words = stopwords.words('english')
             return ' '.join([ps.stem(w) for w in sentence.lower().split() if w not in stop_words])
         # stem only
@@ -88,7 +69,7 @@ def preprocess(sentence: str, remove_punct: bool, remove_stop_words: bool, stemm
     elif lang.lower() == 'ar':
         st = ISRIStemmer()
         # remove punctuations
-        if not remove_punct:
+        if remove_punct:
             sentence = sentence.translate(str.maketrans('', '', punctuation))
         # remove stop words and stem
         if remove_stop_words and stemm:
@@ -105,7 +86,7 @@ def preprocess(sentence: str, remove_punct: bool, remove_stop_words: bool, stemm
         raise Exception('non recognized language please specify either en|ar')
 
     
-class sentence_tranformer():
+class sentence_tranformer_checker():
     def __init__(
         self, 
         targets: Union[list[str], pd.DataFrame], 
@@ -115,20 +96,46 @@ class sentence_tranformer():
         lang: str='en', 
         only_include: list[str]=None,
         encode_batch = 32,
-        encode_target = False
-    ):
+        encode_target = True,
+        remove_punct: bool=True, 
+        remove_stop_words: bool=True, 
+        stemm: bool=False
+    ):    
+        """
+        parameters:
+            source: dataframe or list of input texts to find closest match for.
+            target: dataframe or list of targets text to compare with.
+            model (optional): a sentence tranformer model object, to use instead of the default one, for more [details](https://www.sbert.net/).
+            target_group (optional): goups ids for the target to match only a single target for each group, can either provide list of ids,
+            or the column name in the target dataframe.
+            source_col (partially optional): the source column name used to match, *must be specified for dataframe matching*.
+            target_col (partially optional): the target column name used to match, *must be specified for dataframe matching*.
+            only_include (optional): used only for dataframe matching, allow providing a list of column names to only include for the target matches, provide empty list to get only target_col.
+            encode_batch (optional): the number of sentences to encode in a batch.
+            encode_target: boolean flag to indicate whatever to enocde the targets when initilizing the object (to cache target encoding).
+            lang (optional): the languge of the model ('en'|'ar').
+            remove_punct: boolean flag to indicate whatever to remove punctuations. 
+            remove_stop_words: boolean flag to indicate whatever to remove stop words.
+            stemm: boolean flag to indicate whatever to do stemming.
+        """
+
         self.encode_batch = encode_batch
+        self.remove_punct = remove_punct
+        self.remove_stop_words = remove_stop_words
+        self.stemm = stemm
+        self.lang = lang 
+
         if isinstance(targets, pd.DataFrame):
             self.use_frames = True
     
-            if target_col not in target.columns:
+            if target_col not in targets.columns:
                 raise KeyError('target_col not found in target DataFrame cloumns')
 
             if target_group is not None:
                 if isinstance(target_group, str):
-                    if target_group not in target.columns:
+                    if target_group not in targets.columns:
                         raise KeyError('target_group not found in target DataFrame cloumns')
-                    self.group_ids = target[target_group].tolist()
+                    self.group_ids = targets[target_group].tolist()
                 else:
                     self.group_ids = target_group
             else:
@@ -136,25 +143,28 @@ class sentence_tranformer():
                 
             if only_include is not None:
                 for col_name in only_include:
-                    if col_name not in target.columns:       
+                    if col_name not in targets.columns:       
                         raise KeyError(f'only_include value:({col_name}) not found not found in target DataFrame cloumns')    
                 only_include.insert(0, target_col)
                 
-                target = target.loc[:, only_include]
+                targets = targets.loc[:, only_include]
 
-            self.target_df = target.reset_index(drop=True)
-            self.target_names = target[target_col].tolist()
+            self.target_df = targets.reset_index(drop=True)
+            self.target_names = [preprocess(sent, self.remove_punct, self.remove_stop_words, self.stemm, self.lang) for sent in targets[target_col].tolist()]
         else:
             if isinstance(target_group, list):
                 self.group_ids = target_group
             else:
-                raise TypeError('if target are a list entered groups must also be a text')
+                if target_group is None:
+                    self.group_ids = target_group
+                else:
+                    raise TypeError('if target are a list entered groups must also be a list')
         
-            if not target:
+            if not targets:
                 raise TypeError('Targets are empty') 
 
             self.use_frames = False
-            self.target_names = target
+            self.target_names = [preprocess(sent, self.remove_punct, self.remove_stop_words, self.stemm, self.lang) for sent in targets]
 
         if pd.isnull(self.target_names).any():
             raise ValueError('Targets contain null values')
@@ -177,18 +187,11 @@ class sentence_tranformer():
 
     # used to make sure that if the object targets are updated, the cached targets are deleted
     def __setattr__(self, key, value):
-        self.key = value
+        # self.key = value
         if key == 'target_names' or key == 'target_df':
             if hasattr(self, 'encoded_targets'):
-                del self.encoded_targets
-
-
-    def clean_data(self, remove_punct: bool=True, remove_stop_words: bool=True, stemm: bool=False, lang: str='en', encode_target = False) -> None: 
-        self.source_names = [preprocess(sent, remove_punct, remove_stop_words, stemm, lang) for sent in self.source_names]
-        self.target_names = [preprocess(sent, remove_punct, remove_stop_words, stemm, lang) for sent in self.target_names]
-        if encode_target:
-            print('enocding targets: ')
-            self.encoded_targets = self.model.encode(self.target_names, batch_size=self.encode_batch, show_progress_bar=True,  normalize_embeddings=True) # encode the targets
+                del self.encoded_targets     
+        super().__setattr__(key, value)
 
 
     def match(
@@ -202,6 +205,15 @@ class sentence_tranformer():
     ) -> pd.DataFrame:
         '''
         Main match function. return only the top candidate for every source string.
+        parameters:
+            source: dataframe or list of input texts to find closest match for.
+            source_col (partially optional): the source column name used to match, *must be specified for dataframe matching*.
+            topn: number of matches to return.
+            threshold: the lowest threeshold to ignore matches below it.
+            return_match_idx: return an extra column for each match containing the index of the match within the target_names.
+            batch_size: the size of the batch in inputs to match with targets (to limit space usage).
+        returns:
+            a data frame with 3 columns (source, target, score), and two extra columns for each extra match (target_2, score_2 ...), and an optional extra column for each match containg the match index, if return_match_idxs set to True.
         ''' 
         if isinstance(source, pd.DataFrame) and not hasattr(self, 'target_df'):
             raise TypeError('if target is a dataframe source must also be a dataframe')
@@ -216,17 +228,19 @@ class sentence_tranformer():
                 raise KeyError('source_col not found in source DataFrame cloumns')
 
             self.source_df = source.reset_index(drop=True)
-            self.source_names = source[source_col].tolist()
+            self.source_names = [preprocess(sent, self.remove_punct, self.remove_stop_words, self.stemm, self.lang) for sent in source[source_col].tolist()]
         else:
             if not source:
                 raise TypeError('Inputs are empty')
 
             self.use_frames = False
-            self.source_names = source
+            self.source_names = [preprocess(sent, self.remove_punct, self.remove_stop_words, self.stemm, self.lang) for sent in source]
 
         if not hasattr(self, 'encoded_targets'):
             print('enocding targets: ')
-            self.encoded_targets = self.model.encode(self.target_names, batch_size=self.encode_batch, show_progress_bar=True,  normalize_embeddings=True) # encode the targets
+            encoded_targets = self.model.encode(self.target_names, batch_size=self.encode_batch, show_progress_bar=True,  normalize_embeddings=True) # encode the targets
+        else:
+            encoded_targets = self.encoded_targets
 
         targets = np.full((len(self.source_names), topn), None)
         top_cosine = np.full((len(self.source_names), topn), None)
@@ -236,26 +250,24 @@ class sentence_tranformer():
         for i in tqdm(range(0, len(self.source_names), batch_size)):
             encoded_inputs = self.model.encode(self.source_names[i:i+batch_size], batch_size=self.encode_batch, normalize_embeddings=True) # encode the inputs
 
-            batch_targets, batch_top_cosine, batch_match_idxs = self.max_cosine_sim(encoded_inputs, topn, threshold)
-            start = time()
+            batch_targets, batch_top_cosine, batch_match_idxs = self.max_cosine_sim(encoded_inputs, encoded_targets , topn, threshold)
             targets[i:i+batch_size, :] = batch_targets
             top_cosine[i:i+batch_size, :] = batch_top_cosine
             match_idxs[i:i+batch_size, :] = batch_match_idxs
-            print(f'time taken to concat batch results is {time()-start:.2f}')
         
         match_output = self._make_matchdf(targets, top_cosine, match_idxs, topn, return_match_idx)
 
         return match_output
 
 
-    def max_cosine_sim(self, encoded_inputs,  topn, threshold):
+    def max_cosine_sim(self, encoded_inputs, encoded_targets,  topn, threshold):
         if len(encoded_inputs.shape) == 1:
             a = a.unsqueeze(0)
 
-        if len(self.encoded_targets.shape) == 1:
+        if len(encoded_targets.shape) == 1:
             b = b.unsqueeze(0)
 
-        scores = np.matmul(encoded_inputs, self.encoded_targets.T)
+        scores = np.matmul(encoded_inputs, encoded_targets.T)
 
         if self.group_ids is None:
             max_matches = min((len(self.target_names)-1, topn))
@@ -379,78 +391,6 @@ class sentence_tranformer():
         return match_df
 
 
-
-    def _make_matchdf(self)-> pd.DataFrame:
-        ''' Build dataframe for result return '''
-        if self.use_frames:
-            arr_temp = np.full((len(self.source_names), len(self.target_df.columns)+1), None)
-            for i, (match_idx, score) in enumerate(zip(self.match_idxs_trans[0], self.top_cosine_trans[0])):
-                if match_idx in self.target_df.index:
-                     temp = self.target_df.iloc[match_idx].tolist()
-                     temp.insert(0, score)
-                     arr_temp[i, :] = temp
-            cols = self.target_df.columns.tolist() 
-            cols.insert(0, 'score_1')
-            match_df= pd.DataFrame(arr_temp, columns=cols)
-            # concat targets matches into one dataframe
-            for match_num in range(1, len(self.match_idxs_trans)):
-                arr_temp = np.full((len(self.source_names), len(self.target_df.columns)+1), None)
-                for i, (match_idx, score) in enumerate(zip(self.match_idxs_trans[match_num], self.top_cosine_trans[match_num])):
-                    if match_idx in self.target_df.index:
-                        temp = self.target_df.iloc[match_idx].tolist()
-                        temp.insert(0, score)
-                        arr_temp[i, :] = temp
-                cols = self.target_df.columns.tolist() 
-                cols.insert(0, f'score_{match_num+1}')
-                df_temp= pd.DataFrame(arr_temp, columns=cols)
-                match_df = match_df.merge(df_temp, left_index=True, right_index=True, suffixes=(f'_{match_num}', f'_{match_num+1}'))
-            # merge matches with source
-            match_df = self.source_df.reset_index(drop=True).merge(match_df, left_index=True, right_index=True, suffixes=(f'_source', f'_target'))
-        elif not self.return_match_idx:
-            match_list = []
-            for source, top_scores, targets in zip(self.source_names, self.top_cosine, self.targets):
-                row = []
-                row.append(source)
-                # loop over results of multi matches
-                for top_score, target in zip(top_scores, targets):
-                    row.append(top_score)
-                    row.append(target)
-                     
-                match_list.append(tuple(row))
-
-            # prepare columns names
-            colnames = ['source', 'score', 'prediction']
-            
-            for i in range(2, self.topn+1):
-                colnames.append(f'score_{i}')
-                colnames.append(f'prediction_{i}')
-                
-            match_df = pd.DataFrame(match_list, columns=colnames)
-        else:
-            match_list = []
-            for source, top_scores, targets, match_idxs in zip(self.source_names, self.top_cosine, self.targets, self.match_idxs):
-                row = []
-                row.append(source)
-                # loop over results of multi matches
-                for top_score, target, match_idx in zip(top_scores, targets, match_idxs):
-                    row.append(top_score) 
-                    row.append(target)
-                    row.append(match_idx)
-                match_list.append(tuple(row))
-
-            # prepare columns names
-            colnames = ['source', 'score', 'prediction', 'match_idx']
-            
-            for i in range(2, self.topn+1):
-                colnames.append(f'score_{i}')
-                colnames.append(f'prediction_{i}')
-                colnames.append(f'match_idx_{i}')
-
-            match_df = pd.DataFrame(match_list, columns=colnames)  
-
-        return match_df
-
-
 class word_mover_distance():
     def __init__(self, source_names, target_names, model):
         if not source_names:
